import direct.directbase.DirectStart
from pandac.PandaModules import *
from direct.gui.DirectGui import *
from direct.interval.IntervalGlobal import *
from direct.actor import Actor
from direct.task import Task 
import sys, os, time, math

## import picker 

from interface import *
from headEv1 import *
from headEv2 import *
from headEv3 import *
from headEv4 import *
## from head3d import *

import joystick

import pymedia.audio.sound as sound
import pymedia.audio.acodec as acodec
SAMPLES= 500
BANDS= 10
NUM_FREQS= 4
MOUTH_SPEED=200
DETECT_SENSIBILITY=0.3


OPEN=1
CLOSE=-1


class World(DirectObject):
    
  def __init__(self):
        
    #Set the camera in a fixed position
    # Retitle the main window as the "puppet" 
    wp = WindowProperties()
    wp.setUndecorated(1)
    wp.setTitle('puppet')
    base.win.requestProperties(wp)
        
    # Open three new windows and title them "left", "center", and "right"  
    wp.setTitle('left')
    w2 = base.openWindow(gsg=base.win.getGsg(), props=wp)
        
    wp.setTitle('center')
    w3 = base.openWindow(gsg=base.win.getGsg(), props=wp)
    
    wp.setTitle('right')
    w4 = base.openWindow(gsg=base.win.getGsg(), props=wp)
    
    # position the cameras    
    base.disableMouse()
    ## camera.setPosHpr(-0, -115, 6, 0, 0, 0)             # puppet
    base.camList[0].setPosHpr(-0, -115, 6, 0, 0, 0)             # puppet
    base.camList[1].setPosHpr(-90.1, -3.1, -1, -90, 0, 0)       # left
    ## base.camList[2].setPosHpr(-2.7, -85.2, -0.3, 0, 0, 0)       # center
    base.camList[2].setPosHpr(-2.7, -87.2, -0.3, 0, 0, 0)       # center
    base.camList[3].setPosHpr(90.1, -3.5, 0, 90, 0, 0)          # right

    base.setBackgroundColor( .2, .2, .2 )
    ## self.accept( "space", self.findCam ) # use to locate the right starting position
    self.accept( "escape", self.exit ) # shut down the joystick before closing
    
    # create three nodes that will control which objects cameras will see
    self.puppetSee = render.attachNewNode( "puppetSee" )
    self.enviroSee = self.puppetSee.attachNewNode( "enviroSee" )
    self.allSee = self.enviroSee.attachNewNode( "allSee" )
    
    # setup the left, center, and right cameras to render allSee
    base.camList[0].node().setScene(render)               # puppet
    base.camList[1].node().setScene(self.enviroSee)       # left
    base.camList[2].node().setScene(self.enviroSee)       # center
    base.camList[3].node().setScene(self.enviroSee)       # right
    
    # initialize the mic
    self.initAudio()
    
    #Load the environment
    self.enviroC = loader.loadModelCopy('models/background')
    self.enviroC.reparentTo(self.enviroSee)
    self.enviroC.setPosHprScale(-2,50,0, 0,0,0, 1.3,1.3,1.3)
    self.enviroR = loader.loadModelCopy('models/background')
    self.enviroR.reparentTo(self.enviroSee)
    self.enviroR.setPosHprScale(-52,-2,0, 90,0,0, 1.3,1.3,1.3)
    self.enviroL = loader.loadModelCopy('models/background')
    self.enviroL.reparentTo(self.enviroSee)
    self.enviroL.setPosHprScale(52,-2,0, -90,0,0, 1.3,1.3,1.3)
    
    #Load the actor heads
    ## self.head3D = Head3D(self.allSee)
    ## self.head = self.head3D
    self.headEv1 = HeadEv1(self.allSee)
    self.headEv2 = HeadEv2(self.allSee)
    self.headEv3 = HeadEv3(self.allSee)
    self.headEv4 = HeadEv4(self.allSee)
    
    self.head = self.headEv1
    self.head.showHead()
    
    
    # create a boolean that will controll the automatic or manual control of the jaw
    self.manJaw = False
    
    #Load the interface
    self.Interface = Interface(self.puppetSee)

    self.setupInterfaceAccepts()  # setup messenger accepts from interface.py
    
    
    # check out the render queue - comment out later
    ## print render.ls()
        
    
    ## self.accept( ".", self.teststuff, [ -1 ] )   #### comment out later
    ## self.accept( "/", self.teststuff, [ 1 ] )    #### comment out later
    ## self.accept( ";", self.teststuff2, [ VBase3( -.1, -.1, -.1) ] )   #### comment out later
    ## self.accept( "'", self.teststuff2, [ VBase3( .1, .1, .1) ] )    #### comment out later
    ## self.accept("=", self.head.fadeOutHead )
    ## self.accept("=", self.Interface.fadeOutEye )
    
#########################joystick############################################    

    # setup button event accepts and the event list to be sent to the joystick
    self.setupButtonEvents()
    self.setupHatEvents()
    
    # load and init the joystick
    self.js = joystick.Joy(self.events)
    
    #Add the task to update the head based on the joystick axes
    taskMgr.add(self.MoveHeadTask,"MoveHeadTask")
    
#########################lighting############################################

    self.lightAttrib = LightAttrib.makeAllOff()
    self.ambientLight = AmbientLight( "ambientLight" )
    self.ambientLight.setColor( Vec4( .25, .25, .25, .8 ) )
    self.lightAttrib = self.lightAttrib.addLight( self.ambientLight )
        
    self.directionalLightL = DirectionalLight( "directionalLightL" )
    ## self.directionalLight.setColor( Vec4( .73, .33, .15, 1 ) )
    self.directionalLightL.setColor( Vec4( .73, .73, .73, 1 ) )
    self.directionalLightL.setDirection( Vec3( 100, 100, -200 ) )
    self.directionalLightR = DirectionalLight( "directionalLightR" )
    ## self.directionalLight.setColor( Vec4( .73, .33, .15, 1 ) )
    self.directionalLightR.setColor( Vec4( .73, .73, .73, 1 ) )
    self.directionalLightR.setDirection( Vec3( -100, 100, -200 ) )
    self.lightAttrib = self.lightAttrib.addLight( self.directionalLightL )
    self.lightAttrib = self.lightAttrib.addLight( self.directionalLightR )
        
    ## render.node().setAttrib( self.lightAttrib )        
    self.allSee.node().setAttrib( self.lightAttrib ) 

#########################audio###############################################

  def initAudio(self):
    self.sndInput= sound.Input( 44100, 1, sound.AFMT_S16_LE )
    self.sndInput.start()
          
    self.sndOutput= sound.Output( 44100, 1, sound.AFMT_S16_LE )
          
    self.analyzer= sound.SpectrAnalyzer( 1, SAMPLES, NUM_FREQS )
          
    self.sensibility=DETECT_SENSIBILITY
    self.isSpeaking=False
    self.mouthPose=0
    self.mouthDirection=1
    
    taskMgr.add(self.TaskWingFlapAnim, "Flap Wings or mouth move Task")
    taskMgr.add(self.taskDetectVoice,"taskDetectVoce")
        
    # set up accepts for mic sensibility
    self.accept( "+", self.changeSensibility, [ 0.02 ] )   # increase sensibility
    self.accept( "-", self.changeSensibility, [ -0.02 ] )   # lower sensibility
    
  def changeSensibility(self, value):
    self.sensibility = self.sensibility + value 
    print "change sensibility", self.sensibility
    
  def taskDetectVoice(self, task):
    self.isSpeaking=False 
    s= self.sndInput.getData()
    largestFreqValue= 0
    previousLargestFreqValue= 0
    if s and len( s ):
        freqs=self.analyzer.asFrequencies(s)
        for i in range( 0, len( freqs ) ):
          for j in range( 0, len( freqs[i] ) ):
#===============================================================================
#              freqValue=freqs[i][j]
#              if freqValue > largestFreqValue:
#                   previousLargestFreqValue= largestFreqValue
#                   largestFreqValue= freqValue
#===============================================================================

           if(freqs[i][j]>self.sensibility):
               print "actor just said something, the strength :",freqs[i][j]
               self.isSpeaking=True
               self.mouthDirection=1 # make a mouth open again even while it's closing
              
               
#===============================================================================
#       if(largestFreqValue>0.2):
#           print largestFreqValue 
#===============================================================================
    return Task.cont

  def TaskWingFlapAnim(self, task):
      
    dt = globalClock.getDt()  
            
    if(self.mouthDirection==1):
        if(self.mouthPose>30):
            self.mouthDirection=-1
    else:
        if(self.mouthPose<1):    
            self.mouthDirection=1
        
    if(self.isSpeaking==True):
        self.mouthPose=self.mouthPose+self.mouthDirection*MOUTH_SPEED*dt
    else:
        ## ## if( (self.mouthPose>0 and self.mouthDirection == 1) or (self.mouthPose>4 and self.mouthDirection == -1)):
        if( self.mouthPose>1 ):
        ## self.mouthDirection = -1  # if not speaking then start closing the mouth even if it was opening
        ## if (self.mouthPose>4 ):
            self.mouthPose=self.mouthPose+self.mouthDirection*MOUTH_SPEED*dt
        else:
            self.mouthPose=0
                
    ## self.whiteBird.pose('flap',self.mouthPose)   

    ## self.jaw.setR(self.mouthPose * -1)
    # if not currently controlling the jaw manually and mouthPose > 0 -- don't set a pose if 
    # not speaking because that resets the face and does not allow other emotions to function
    if (self.manJaw == False and self.mouthPose > 0):  
        self.facePoseTalking(self.mouthPose * -1)
        print 'in task', self.mouthPose, self.mouthPose * -1

    # print self.mouthPose, self.isSpeaking

    return Task.cont
  

#########################face posing#########################################
   
  def moveEyes(self):
    eyeLCenterX = 0.644
    eyeLEdge = .55
    eyeRCenterX = 0.41
    eyeCenterY = 0.372
    mPos = base.mouseWatcherNode.getMouse()
    mX = mPos.getX()
    mY = mPos.getY()
    print mX, mY
    if (mX < eyeLEdge):
        # rotate charlie's eyes based on right eye ball
        rotX = mX - eyeRCenterX
    else:
        rotX = mX - eyeLCenterX
    # rotate charlie's eyes 
    self.head.eyesRotateLR(rotX * 450)
    self.head.eyesRotateUD((mY - eyeCenterY) * 225)
    # rotate the interface's eyes           
    self.Interface.moveIntEyes(rotX * 450, (mY - eyeCenterY) * 225 * -1)
    
  def charlieSwitch(self, which):
    #switch between the different Charlie Evolutions
    if (which == 1):
        self.head.hideHead()
        self.head = self.headEv1
        self.head.showHead()
        self.Interface.resetIntEyes()
    elif (which == 2):
        self.head.hideHead()
        self.head = self.headEv2
        self.head.showHead()
        self.Interface.resetIntEyes()
    elif (which == 3):
        self.head.hideHead()
        self.head = self.headEv3
        self.head.showHead()
        self.Interface.resetIntEyes()
    elif (which == 4):
        self.head.hideHead()
        self.head = self.headEv4
        self.head.showHead()
        self.Interface.resetIntEyes()
        
#############################################################################
    

######################joystick###############################################

  def MoveHeadTask(self,task):
    # read the joystick 
    axis0 = self.js.getAxis(0,False)
    axis1 = self.js.getAxis(1,False)
    axis3 = self.js.getAxis(3,False)
    #Tilt the head side to side
    if (axis0 > 0.5 or axis0 < -0.5):
        self.head.headRotateP(axis0)
    #Look up or down
    if (axis1 > 0.5 or axis1 < -0.5):
        self.head.headRotateR(axis1)
    #Turn the head right and left
    if (axis3 > 0.5 or axis3 < -0.5):
        self.head.headRotateH(axis3)
    return Task.cont

  def MoveEyes(self):
    # read the hat 
    hat0 = self.js.getHat(0)
    ## print hat0
    #Look up or down
    if (hat0[1] > 0.3 or hat0[1] < -0.3):
        self.head.eyesRotateUD( 20 * hat0[1] ) # eyes rotateU
    #Look right and left
    if (hat0[0] > 0.3 or hat0[0] < -0.3):
        self.head.eyesRotateLR( 30 * hat0[0] )
    # Look straight ahead
    if (hat0[0] == 0 and hat0[1] == 0) :
        self.head.eyesReset()
        self.Interface.resetIntEyes()
    return Task.cont

  def setupHatEvents(self):
    # setup hat event accepts 
    self.accept("hatevent",self.MoveEyes)

  def setupButtonEvents(self):
    # setup button event accepts and the event list to be sent to the joystick
    self.events = {}
    self.accept("joyButton0-up",self.ButtonHit, [0, True])
    self.accept("joyButton0-down",self.ButtonHit, [0, False])
    self.events[0] = "joyButton0"
    self.accept("joyButton1-up",self.ButtonHit, [1, True])
    self.accept("joyButton1-down",self.ButtonHit, [1, False])
    self.events[1] = "joyButton1"
    self.accept("joyButton2-up",self.ButtonHit, [2, True])
    self.accept("joyButton2-down",self.ButtonHit, [2, False])
    self.events[2] = "joyButton2"
    self.accept("joyButton3-up",self.ButtonHit, [3, True])
    self.accept("joyButton3-down",self.ButtonHit, [3, False])
    self.events[3] = "joyButton3"
    self.accept("joyButton4-up",self.ButtonHit, [4, True])
    self.accept("joyButton4-down",self.ButtonHit, [4, False])
    self.events[4] = "joyButton4"
    self.accept("joyButton5-up",self.ButtonHit, [5, True])
    self.accept("joyButton5-down",self.ButtonHit, [5, False])
    self.events[5] = "joyButton5"

  def ButtonHit(self,button,up):
    # handle joystick button hits
    #Button 0 
    if (button == 0):
        if (up):  # UP
            # close the mouth
            print "button0 up"
            ## self.jawRotate( 0 )
            self.head.facePoseN()
            self.manJaw = False  # mic now has control of the jaw 
        else:     # DOWN
            # open mouth to talk
            print "button0 down"
            ## self.jawRotate( -30 )
            self.head.facePoseTalking(-30)
            self.manJaw = True  # joystick now has control of the jaw over the mic
    #Button 1 
    elif (button == 1):
        if (up):  # UP
            print "button1 up"
        else:     # DOWN
            print "button1 down"
    #Button 2 
    elif (button == 2):
        if (up):  # UP
            print "button2 up"
        else:     # DOWN
            print "button2 down"
    #Button 3 
    elif (button == 3):
        if (up):  # UP
            print "button3 up"
            self.head.headReset('finish')
        else:     # DOWN
            print "button3 down"
            self.head.headReset('start')
    #Button 4 
    elif (button == 4):
        if (up):  # UP
            # open the eyes
            print "button4 up"
            self.head.eyeLidsRotate(0)
        else:     # DOWN
            # close the eyes
            print "button4 down"
            self.head.eyeLidsRotate(-30)
    #Button 5 
    elif (button == 5):
        if (up):  # UP
            # open the eyes - alternitive button
            print "button5 up"
            self.head.eyeLidsRotate(0)
        else:     # DOWN
            # close the eyes - alternitive button
            print "button5 down"
            self.head.eyeLidsRotate(-30)
 
#################Interface buttons###########################################
  
  def setupInterfaceAccepts(self):
    ## ## self.accept('gotHappyButton', self.facePoseHappy)  
    ## ## self.accept('gotWorryButton', self.facePoseWorried)
    ## ## self.accept('gotMadButton', self.facePoseMad)
    ## ## self.accept('gotSurprisedButton', self.facePoseSurprised)
    ## ## self.accept('gotSadButton', self.facePoseSad)
    ## ## self.accept('gotSleepyButton', self.facePoseSleepy)
    ## ## self.accept('gotNButton', self.facePoseN)
    
    ## self.accept('gotSleepButton', self.faceGotoSleep)
    ## self.accept('gotWakeupButton', self.faceWakeup)
    self.accept('gotEv1CharlieButton', self.charlieSwitch, [ 1 ])
    self.accept('gotEv2CharlieButton', self.charlieSwitch, [ 2 ])
    self.accept('gotEv3CharlieButton', self.charlieSwitch, [ 3 ])
    self.accept('gotEv4CharlieButton', self.charlieSwitch, [ 4 ])
    self.accept('gotMoveEyes', self.moveEyes)
    ## self.accept('gotMoveEyes', self.moveEyes)
    self.accept('gotMicUpButton', self.changeSensibility, [ 0.02 ])    # increase sensibility
    self.accept('gotMicDownButton', self.changeSensibility, [ -0.02 ]) # lower sensibility
     

#############################################################################
  
  def exit(self):
    # clear out the event queue
    self.js.quitJoy()
    print 'bye'
    sys.exit
       
  def teststuff(self, value):
    ## self.radarCamera.setY(self.radarCamera.getY() + 1)
    self.enviroR.setY(self.enviroR.getY() + value)
    ## self.headEv1.detachNode()
    ## self.headEv1.removeNode()
    print self.enviroR.getY()
    ## print 'in test stuff'
    
  def teststuff2(self, value):
    ## self.radarCamera.setY(self.radarCamera.getY() + 1)
    self.enviroR.setScale(self.enviroR.getScale() + value)
    ## print self.radarPlane.getX()
    print self.enviroR.getScale()
    ## print 'in test stuff'
    
  def findCam(self):
    print base.camera.getPos(), base.camera.getHpr() 

w = World()
run()