import direct.directbase.DirectStart
from pandac.PandaModules import *
from direct.gui.DirectGui import *
from direct.interval.IntervalGlobal import *
from direct.actor import Actor
from direct.task import Task 
import sys, os, time, math


class HeadEv2(DirectObject):
    
  def __init__(self, parent):
        
    #Save the intended parent
    self.parent = parent 
    
    #Load the head actor
    self.head = Actor.Actor('models/headEv2',
      {'test' : 'models/testEv2'} )
    self.head.setPosHprScale(0,0, -2,0,0,0, .1,.1,.1)
    ## self.head.reparentTo(self.parent)
    
    #Expose the joints needed to animate the head
    self.headRot = self.head.controlJoint(None, 'modelRoot', 'BoneHeadRot')
    self.jaw = self.head.controlJoint(None, 'modelRoot', 'BoneCJawRot')
    self.browR = self.head.controlJoint(None, 'modelRoot', 'BoneCBrowUpRotR')
    self.browL = self.head.controlJoint(None, 'modelRoot', 'BoneCBrowUpRotL')
    self.eyeR = self.head.controlJoint(None, 'modelRoot', 'BoneEyeRotR')
    self.eyeL = self.head.controlJoint(None, 'modelRoot', 'BoneEyeRotL')
    self.eyeLidUpR = self.head.controlJoint(None, 'modelRoot', 'BoneCEyeLidRotR')
    self.eyeLidUpL = self.head.controlJoint(None, 'modelRoot', 'BoneCEyeLidRotL')
    self.eyeLidLowR = self.head.controlJoint(None, 'modelRoot', 'BoneCEyeRimRotR')
    self.eyeLidLowL = self.head.controlJoint(None, 'modelRoot', 'BoneCEyeRimRotL')
    self.cheaks = self.head.controlJoint(None, 'modelRoot', 'BoneCCheaks')
    self.nose = self.head.controlJoint(None, 'modelRoot', 'BoneCNose')
    self.lipUp = self.head.controlJoint(None, 'modelRoot', 'BoneCLipUpRot')
    self.lipLow = self.head.controlJoint(None, 'modelRoot', 'BoneCLipLowRot')
    self.lipCorners = self.head.controlJoint(None, 'modelRoot', 'BoneCLipCorners')
    
    # create an interval to be used to get the head back into a neutral orientation
    self.headResetInt = LerpHprInterval(self.headRot, 30, VBase3(0,0,0), startHpr = self.headRot.getHpr())
    
    # check out the render queue - comment out later
    ## print render.ls()
        
    # setup the keyboard for testing of individual bones
    ## self.testAccepts()         #### comment out later
    
    # run the default animation so that all bones will come under control
    self.testHead()
    
    # start head off in a neutral pose
    self.facePoseN()
    
    # setup messenger accepts from interface.py
    self.setupInterfaceAccepts()  
    
    ## self.accept( ".", self.teststuff, [ -1 ] )   #### comment out later
    ## self.accept( "/", self.teststuff, [ 1 ] )    #### comment out later
    ## self.accept( "k", self.teststuff2, [ VBase3( 0, -.01, 0) ] )   #### comment out later
    ## self.accept( "j", self.teststuff2, [ VBase3( 0, .01, 0) ] )    #### comment out later
        

#########################face posing#########################################

  def testAccepts(self):
    #Setup keyboard control for the playing of the animations
    self.accept( "w", self.testHead )
    self.accept( "e", self.jawRotate, [ -10,1 ] )   # open jaw
    self.accept( "r", self.jawRotate, [ 10,1 ] )    # close jaw
    self.accept( "t", self.browRotateR, [ 10,1 ] )  # raise browR
    self.accept( "y", self.browRotateR, [ -10,1 ] ) # lower browR
    self.accept( "u", self.browRotateL, [ 10,1 ] )  # raise browL
    self.accept( "i", self.browRotateL, [ -10,1 ] ) # lower browL
    self.accept( "o", self.browTwistR, [ -10,1 ] )  # twist up browR
    self.accept( "p", self.browTwistR, [ 10,1 ] )   # twist down browR
    self.accept( "[", self.browTwistL, [ 10,1 ] )   # twist up browL
    self.accept( "]", self.browTwistL, [ -10,1 ] )  # twist down browL
    self.accept( "a", self.eyesRotateLR, [ 10,1 ] ) # eyes rotateR
    self.accept( "s", self.eyesRotateLR, [ -10,1 ] )# eyes rotateL
    self.accept( "d", self.eyesRotateUD, [ 10,1 ] ) # eyes rotateU
    self.accept( "f", self.eyesRotateUD, [ -10,1 ] )# eyes rotateD
    self.accept( "g", self.eyeLidsRotate, [ -10,1 ] ) # eye lids open
    self.accept( "h", self.eyeLidsRotate, [ 10,1 ] ) # eye lids colse
    self.accept( "j", self.cheaksRotate, [ -10,1 ] ) # cheaks raise
    self.accept( "k", self.cheaksRotate, [ 10,1 ] ) # cheaks lower
    self.accept( "l", self.noseRotate, [ -10,1 ] ) # nose raise
    self.accept( ";", self.noseRotate, [ 10,1 ] ) # nose lower
    self.accept( "z", self.lipUpRotate, [ -10,1 ] ) # lip upper raise
    self.accept( "x", self.lipUpRotate, [ 10,1 ] ) # lip upper lower
    self.accept( "c", self.lipLowRotate, [ -10,1 ] ) # lip lower raise
    self.accept( "v", self.lipLowRotate, [ 10,1 ] ) # lip lower lower
    self.accept( "b", self.lipCornersRotate, [ -10,1 ] ) # lip corners raise
    self.accept( "n", self.lipCornersRotate, [ 10,1 ] ) # lip corners lower
    self.accept( "m", self.lipCornersScale, [ VBase3(-.1, -.1, -.1),1 ] ) # lip corners raise
    self.accept( ",", self.lipCornersScale, [ VBase3(.1, .1, .1),1 ] ) # lip corners lower
    
    self.accept( "enter", self.findFacePose ) # use to locate the right starting position
    
  def testHead(self):
    self.head.play('test')
    
  def showHead(self):
    self.head.reparentTo(self.parent)
    self.facePoseN()
    ## self.head.show()
    
  def hideHead(self):
    self.head.detachNode()
    ## self.head.hide()
    
  def fadeOutHead(self):
    alphaInterval = LerpFunctionInterval(self.head.setAlphaScale, toData = 0.0, fromData = 1.0, duration = 1.2)
    alphaInterval.start()
    
  def jawRotate(self, value, test = False):
    if test:
        self.jaw.setR(min([ max([ self.jaw.getR() + value, -30 ]), 0 ]))
    else:
        self.jaw.setR(value)
    print self.jaw.getR()
    
  def browRotateR(self, value, test = False):
    if test:
        self.browR.setR(min([ max([ self.browR.getR() + value, -10 ]), 30 ]))
    else:
        self.browR.setR(value)
    ## print self.browR.getR()
    
  def browRotateL(self, value, test = False):
    if test:
        self.browL.setR(min([ max([ self.browL.getR() + value, -10 ]), 30 ]))
    else:
        self.browL.setR(value)
    ## print self.browL.getR()
    
  def browTwistR(self, value, test = False):
    if test:
        self.browR.setP(min([ max([ self.browR.getP() + value, -30 ]), 50 ]))
    else:
        self.browR.setP(value)
    ## print self.browR.getR()
    
  def browTwistL(self, value, test = False):
    if test:
        self.browL.setP(min([ max([ self.browL.getP() + value, -50 ]), 30 ]))
    else:
        self.browL.setP(value)
    ## print self.browL.getR()
    
  def eyesRotateLR(self, value, test = False):
    if test:
        self.eyeR.setH(min([ max([ self.eyeR.getH() + value, -30 ]), 30 ]))
        self.eyeL.setH(min([ max([ self.eyeL.getH() + value, -30 ]), 30 ]))
    else:
        self.eyeR.setH(value)
        self.eyeL.setH(value)
    ## print self.eyeR.getP(), self.eyeL.getP()

  def eyesRotateUD(self, value, test = False):
    if test:
        self.eyeR.setR(min([ max([ self.eyeR.getR() + value, -30 ]), 10 ]))
        self.eyeL.setR(min([ max([ self.eyeL.getR() + value, -30 ]), 10 ]))
    else:
        self.eyeR.setR(value)
        self.eyeL.setR(value)
    ## print self.eyeR.getR(), self.eyeL.getR()
    
  def eyesReset(self):
    self.eyeR.setR(0)
    self.eyeR.setH(0)
    self.eyeL.setR(0)
    self.eyeL.setH(0)
    ## print self.eyeR.getR(), self.eyeL.getR(), self.eyeR.getH(), self.eyeL.getH()
    
  def eyeLidsRotate(self, value, test = False):
    if test:
        self.eyeLidUpR.setR(min([ max([ self.eyeLidUpR.getR() + value, -30 ]), 30 ]))
        self.eyeLidUpL.setR(min([ max([ self.eyeLidUpL.getR() + value, -30 ]), 30 ]))
        self.eyeLidLowR.setR(min([ max([ self.eyeLidLowR.getR() + value*-1, -30 ]), 30 ]))
        self.eyeLidLowL.setR(min([ max([ self.eyeLidLowL.getR() + value*-1, -30 ]), 30 ]))
    else:
        self.eyeLidUpR.setR(value)
        self.eyeLidUpL.setR(value)
        self.eyeLidLowR.setR(value*-1)
        self.eyeLidLowL.setR(value*-1)
    ## print self.eyeR.getR(), self.eyeL.getR()
    
  def cheaksRotate(self, value, test = False):
    if test:
        self.cheaks.setR(min([ max([ self.cheaks.getR() + value, -30 ]), 30 ]))
    else:
        self.cheaks.setR(value)
    ## print self.cheaks.getR()

  def noseRotate(self, value, test = False):
    if test:
        self.nose.setR(min([ max([ self.nose.getR() + value, -20 ]), 20 ]))
    else:
        self.nose.setR(value)
    ## print self.nose.getR()
    
  def lipUpRotate(self, value, test = False):
    if test:
        self.lipUp.setR(min([ max([ self.lipUp.getR() + value, -50 ]), 50 ]))
    else:
        self.lipUp.setR(value)
    ## print self.lipUp.getR()
    
  def lipLowRotate(self, value, test = False):
    if test:
        self.lipLow.setR(min([ max([ self.lipLow.getR() + value, -50 ]), 50 ]))
    else:
        self.lipLow.setR(value)
    ## print self.lipLow.getR()
    
  def lipCornersRotate(self, value, test = False):
    if test:
        self.lipCorners.setR(min([ max([ self.lipCorners.getR() + value, -80 ]), 50 ]))
    else:
        self.lipCorners.setR(value)
    ## print self.lipCorners.getR()
    
  def lipCornersScale(self, value, test = False):
    if test:
        self.lipCorners.setScale(self.lipCorners.getScale() + value)
    else:
        self.lipCorners.setScale(value)
    ## print self.lipCorners.getScale()
    
  def findFacePose(self):
    print "************Here starts a face pose printout*********************"
    print "jaw R:", self.jaw.getR()  
    print "browR R:", self.browR.getR()  
    print "browL R:", self.browL.getR()  
    print "browR P:", self.browR.getP()  
    print "browL P:", self.browL.getP()
    print "eyes UD R:", self.eyeR.getR()    
    print "eyes LR H:", self.eyeR.getH()    
    print "eyeLidsUp R:", self.eyeLidUpR.getR()    
    print "cheaks R:", self.cheaks.getR()  
    print "nose R:", self.nose.getR()  
    print "lipUp R:", self.lipUp.getR()  
    print "lipLow R:", self.lipLow.getR()  
    print "lipCorners R:", self.lipCorners.getR() 
    print "lipCorners Scale:", self.lipCorners.getScale() 
    
  def facePose(self, pose, jR, bRR, bRL, bTR, bTL, eRUD, eRLR, eLR, cR, nR, lUR, lLR, lCR, lCS ):
    print "I'm " + pose
    self.jawRotate(jR) 
    self.browRotateR(bRR)  
    self.browRotateL(bRL)    
    self.browTwistR(bTR)    
    self.browTwistL(bTL)    
    self.eyesRotateUD(eRUD)
    self.eyesRotateLR(eRLR)
    self.eyeLidsRotate(eLR) 
    self.cheaksRotate(cR)
    self.noseRotate(nR)  
    self.lipUpRotate(lUR)
    self.lipLowRotate(lLR)
    self.lipCornersRotate(lCR)
    self.lipCornersScale(lCS)
    
  def facePoseN(self):
    # call facePose with 
    #               pose,   jR, bRR, bRL,bTR,bTL, eRUD, eRLR, eLR, cR,  nR,  lUR,lLR,lCR, lCS 
    self.facePose("neutral", 0, -10, -10, 0, 10, -0.58,   0,  0, 1.35, 1.7, -10, 20, 0,  VBase3(.9,.9,.9))
    
  def facePoseHappy(self):
    # call facePose with 
    #               pose, jR, bRR, bRL,bTR,bTL, eRUD, eRLR, eLR, cR,  nR, lUR,lLR,lCR, lCS 
    self.facePose("happy", 0,  30, 30, 10, -10, -0.57,  0,   10, 30, -10, 10, 20, 30,  VBase3(1.1,1.1,1.1))
    
  def facePoseMad(self):
    # call facePose with 
    #               pose, jR, bRR, bRL,bTR,bTL, eRUD, eRLR, eLR, cR,  nR,  lUR,lLR, lCR, lCS 
    self.facePose("mad",   0, -10, -10, 50,-50,-0.58,   0,  0,  11.35, 0,  0, 30, -20, VBase3(.8,.8,.8))
    
  def facePoseSurprised(self):
    # call facePose with 
    #               pose,     jR, bRR, bRL, bTR, bTL, eRUD, eRLR, eLR, cR,  nR, lUR,lLR,lCR, lCS 
    self.facePose("surprised", 0,  30, 30, -10, 10, -0.57,   0,   30, 30, -10, 20, 10, 10,  VBase3(.7,.7,.7))
    
  def facePoseSad(self):
    # call facePose with 
    #               pose, jR, bRR, bRL,bTR,bTL, eRUD, eRLR, eLR,   cR, nR,lUR,lLR, lCR, lCS 
    self.facePose("sad",   0, -10, -10,-30, 30,-0.57,    0,   0,-18.6, 20,  0, 20, -50,  VBase3(.8,.8,.8))
    
  def facePoseSleepy(self):
    # call facePose with 
    #               pose,  jR,  bRR,  bRL, bTR,bTL, eRUD, eRLR, eLR, cR, nR,  lUR,lLR,lCR, lCS 
    self.facePose("sleepy", -10, -10, -10, 0,   10, -0.57,   0, -10, -20, 20, -10, 30, -20, VBase3(.7,.7,.7))
    
  def facePoseWorried(self):
    # call facePose with 
    #               pose,   jR, bRR, bRL,bTR,bTL, eRUD, eRLR, eLR, cR, nR, lUR,lLR,lCR, lCS 
    self.facePose("worried", 0, 30,  30, 50, -50, -0.57,   0,  20, 20, 0, -10, 10, -10, VBase3(1.1,1.1,1.1))

  def facePoseTalking(self,mp):
    #move the lower lip based on the amount of jaw rotation
    lp = 60 + mp
    # call facePose with 
    #               pose,   jR,  bRR, bRL,bTR,bTL, eRUD, eRLR, eLR, cR,  nR,  lUR,lLR,lCR, lCS 
    self.facePose("talking", mp, -10, -10, 0, 10, -0.58,   0,  0, 1.35, 1.7, -10, lp, 0,  VBase3(.7,.7,.7))
    
  def headRotateH(self, axis):
    #Turn the head right and left
    self.headRot.setH(min([ max([ self.headRot.getH() + 2 * axis, -80 ]), 80 ]))
    self.headRot.setY(min([ max([ self.headRot.getY() - 2 * axis, -75 ]), 80 ]))
    
  def headRotateP(self, axis):
    #Tilt the head side to side
    self.headRot.setP(min([ max([ self.headRot.getP() + 2 * axis, -30 ]), 30 ]))
    self.headRot.setY(min([ max([ self.headRot.getY() - 2.5 * axis, -30 ]), 30 ]))
    
  def headRotateR(self, axis):
    #Look up or down
    self.headRot.setR(min([ max([ self.headRot.getR() + 2 * axis, -30 ]), 50 ]))
    self.headRot.setX(min([ max([ self.headRot.getX() + 3 * axis, -165 ]), -45 ]))
    
  def headReset(self, dir):
    if (dir == 'start'):
        self.headResetInt.start()
        self.headRot.setX(-121)
        self.headRot.setY(0)
    else:
        self.headResetInt.finish()
    
#############################################################################


 
#################Interface buttons###########################################
  
  def setupInterfaceAccepts(self):
    self.accept('gotHappyButton', self.facePoseHappy)  
    self.accept('gotWorryButton', self.facePoseWorried)
    self.accept('gotMadButton', self.facePoseMad)
    self.accept('gotSurprisedButton', self.facePoseSurprised)
    self.accept('gotSadButton', self.facePoseSad)
    self.accept('gotSleepyButton', self.facePoseSleepy)
    self.accept('gotNButton', self.facePoseN)
    ## self.accept('gotMoveEyes', self.moveEyes)
    ## self.accept('gotMoveEyes', self.moveEyes)
     

#############################################################################
  
       
  def teststuff(self, value):
    ## self.radarCamera.setY(self.radarCamera.getY() + 1)
    self.rightEyeBall.setZ(self.rightEyeBall.getZ() + value)
    print self.rightEyeBall.getZ()
    ## print 'in test stuff'
    
  def teststuff2(self, value):
    ## self.radarCamera.setY(self.radarCamera.getY() + 1)
    self.head.setScale(self.head.getScale() + value)
    ## print self.radarPlane.getX()
    print self.head.getScale()
    ## print 'in test stuff'
    
